@@ -49,6 +49,7 @@ import javax.servlet.http.HttpServletResponse;
 import javax.ws.rs.Consumes;
 import javax.ws.rs.Produces;
 import javax.ws.rs.WebApplicationException;
+import javax.ws.rs.core.Application;
 import javax.ws.rs.core.Cookie;
 import javax.ws.rs.core.HttpHeaders;
 import javax.ws.rs.core.MediaType;
@@ -75,6 +76,8 @@ import org.apache.cxf.jaxrs.ext.MessageContextImpl;
 import org.apache.cxf.jaxrs.ext.ProtocolHeaders;
 import org.apache.cxf.jaxrs.ext.ProtocolHeadersImpl;
 import org.apache.cxf.jaxrs.ext.multipart.MultipartBody;
+import org.apache.cxf.jaxrs.ext.search.SearchContext;
+import org.apache.cxf.jaxrs.ext.search.SearchContextImpl;
 import org.apache.cxf.jaxrs.impl.HttpHeadersImpl;
 import org.apache.cxf.jaxrs.impl.HttpServletResponseFilter;
 import org.apache.cxf.jaxrs.impl.MetadataMap;
@@ -89,6 +92,7 @@ import org.apache.cxf.jaxrs.model.OperationResourceInfo;
 import org.apache.cxf.jaxrs.model.OperationResourceInfoComparator;
 import org.apache.cxf.jaxrs.model.Parameter;
 import org.apache.cxf.jaxrs.model.ParameterType;
+import org.apache.cxf.jaxrs.model.ProviderInfo;
 import org.apache.cxf.jaxrs.model.URITemplate;
 import org.apache.cxf.jaxrs.provider.AbstractConfigurableProvider;
 import org.apache.cxf.jaxrs.provider.ProviderFactory;
@@ -107,9 +111,8 @@ public final class JAXRSUtils {
     
     private static final Logger LOG = LogUtils.getL7dLogger(JAXRSUtils.class);
     private static final ResourceBundle BUNDLE = BundleUtils.getBundle(JAXRSUtils.class);
-    private static final String PROPOGATE_EXCEPTION = "org.apache.cxf.propogate.exception";
-    private static final String FORM_PARAM_MAP = JAXRSUtils.class.getName() + ".FORM_DATA";
-
+    private static final String PROPAGATE_EXCEPTION = "org.apache.cxf.propagate.exception";
+    
     private JAXRSUtils() {        
     }
     
@@ -181,11 +184,15 @@ public final class JAXRSUtils {
     }
     
     @SuppressWarnings("unchecked")
-    public static void handleSetters(OperationResourceInfo ori,
-                                     Object requestObject,
-                                     Message message) {
+    public static void injectParameters(OperationResourceInfo ori,
+                                        Object requestObject,
+                                        Message message) {
         ClassResourceInfo cri = ori.getClassResourceInfo();
-        InjectionUtils.injectContextMethods(requestObject, cri, message);
+                
+        if (cri.isSingleton() 
+            && (!cri.getParameterMethods().isEmpty() || !cri.getParameterFields().isEmpty())) {
+            LOG.fine("Injecting request parameters into singleton resource is not thread-safe");
+        }
         // Param methods
         MultivaluedMap<String, String> values = 
             (MultivaluedMap<String, String>)message.get(URITemplate.TEMPLATE_PARAMETERS);
@@ -194,12 +201,11 @@ public final class JAXRSUtils {
             Object o = createHttpParameterValue(p, 
                                                 m.getParameterTypes()[0],
                                                 m.getGenericParameterTypes()[0],
+                                                m.getParameterAnnotations()[0],
                                                 message,
                                                 values,
                                                 ori);
-            if (o != null) { 
-                InjectionUtils.injectThroughMethod(requestObject, m, o);
-            }
+            InjectionUtils.injectThroughMethod(requestObject, m, o);
         }
         // Param fields
         for (Field f : cri.getParameterFields()) {
@@ -207,12 +213,11 @@ public final class JAXRSUtils {
             Object o = createHttpParameterValue(p, 
                                                 f.getType(),
                                                 f.getGenericType(),
+                                                f.getAnnotations(),
                                                 message,
                                                 values,
                                                 ori);
-            if (o != null) { 
-                InjectionUtils.injectFieldValue(f, requestObject, o);
-            }
+            InjectionUtils.injectFieldValue(f, requestObject, o);
         }
         
     }
@@ -221,10 +226,12 @@ public final class JAXRSUtils {
                                                  String path, 
                                                  MultivaluedMap<String, String> values,
                                                  Message message) {
-        
-        LOG.fine(new org.apache.cxf.common.i18n.Message("START_CRI_MATCH", 
+        boolean isFineLevelLoggable = LOG.isLoggable(Level.FINE); 
+        if (isFineLevelLoggable) {
+            LOG.fine(new org.apache.cxf.common.i18n.Message("START_CRI_MATCH", 
                                                         BUNDLE, 
                                                         path).toString());
+        }
         if (resources.size() == 1) { 
             return resources.get(0).getURITemplate().match(path, values)
                    ? resources.get(0) : null;
@@ -238,13 +245,15 @@ public final class JAXRSUtils {
             MultivaluedMap<String, String> map = new MetadataMap<String, String>();
             if (cri.getURITemplate().match(path, map)) {
                 candidateList.put(cri, map);
-                LOG.fine(new org.apache.cxf.common.i18n.Message("CRI_SELECTED_POSSIBLY", 
+                if (isFineLevelLoggable) {
+                    LOG.fine(new org.apache.cxf.common.i18n.Message("CRI_SELECTED_POSSIBLY", 
                                                                 BUNDLE, 
                                                                 cri.getServiceClass().getName(),
                                                                 path, 
                                                                 cri.getURITemplate().getValue()).toString());
-            } else {
-                LOG.fine(new org.apache.cxf.common.i18n.Message("NO_CRI_MATCH", 
+                }
+            } else if (isFineLevelLoggable) {
+                LOG.fine(new org.apache.cxf.common.i18n.Message("CRI_NO_MATCH", 
                                                                 BUNDLE, 
                                                                 path,
                                                                 cri.getServiceClass().getName()).toString());
@@ -256,10 +265,12 @@ public final class JAXRSUtils {
                 candidateList.entrySet().iterator().next();
             values.putAll(firstEntry.getValue());
             ClassResourceInfo cri = firstEntry.getKey();
-            LOG.fine(new org.apache.cxf.common.i18n.Message("CRI_SELECTED", 
+            if (isFineLevelLoggable) {
+                LOG.fine(new org.apache.cxf.common.i18n.Message("CRI_SELECTED", 
                                                          BUNDLE, 
                                                          cri.getServiceClass().getName(),
                                                          path, cri.getURITemplate().getValue()).toString());
+            }
             return cri;
         }
         
@@ -273,7 +284,8 @@ public final class JAXRSUtils {
                                                          String requestContentType, 
                                                          List<MediaType> acceptContentTypes,
                                                          boolean logNow) {
-        if (LOG.isLoggable(Level.FINE)) {
+        boolean isFineLevelLoggable = LOG.isLoggable(Level.FINE); 
+        if (isFineLevelLoggable) {
             org.apache.cxf.common.i18n.Message msg = 
                 new org.apache.cxf.common.i18n.Message("START_OPER_MATCH", 
                                                        BUNDLE,
@@ -330,7 +342,7 @@ public final class JAXRSUtils {
                             logNoMatchMessage(ori, path, httpMethod, requestType, acceptContentTypes);
                         }
                     }
-                    if (added && LOG.isLoggable(Level.FINE)) {
+                    if (added && isFineLevelLoggable) {
                         LOG.fine(new org.apache.cxf.common.i18n.Message("OPER_SELECTED_POSSIBLY", 
                                   BUNDLE, 
                                   ori.getMethodToInvoke().getName()).toString());
@@ -354,9 +366,11 @@ public final class JAXRSUtils {
                          BUNDLE, resource.getServiceClass().getName(), 
                          ori.getMethodToInvoke().getName()).toString());
             }
-            LOG.fine(new org.apache.cxf.common.i18n.Message("OPER_SELECTED", 
-                           BUNDLE, ori.getMethodToInvoke().getName(), 
-                           resource.getServiceClass().getName()).toString());
+            if (isFineLevelLoggable) {
+                LOG.fine(new org.apache.cxf.common.i18n.Message("OPER_SELECTED", 
+                               BUNDLE, ori.getMethodToInvoke().getName(), 
+                               resource.getServiceClass().getName()).toString());
+            }
             return ori;
         }
         
@@ -530,11 +544,16 @@ public final class JAXRSUtils {
         for (int i = 0; i < parameterTypes.length; i++) {
             Class<?> param = parameterTypes[i]; 
             Type genericParam = genericParameterTypes[i];
-            if (param == Object.class && genericParam instanceof TypeVariable) {
+            if (genericParam instanceof TypeVariable) {
                 genericParam = InjectionUtils.getSuperType(ori.getClassResourceInfo().getServiceClass(), 
                                                            (TypeVariable)genericParam);
-                param = (Class)genericParam; 
             }
+            if (param == Object.class) {
+                param = (Class)genericParam; 
+            } else if (genericParam == Object.class) {
+                genericParam = param;
+            }
+            
             
             Object paramValue = processParameter(param, 
                                                  genericParam,
@@ -585,6 +604,7 @@ public final class JAXRSUtils {
             return createHttpParameterValue(parameter,
                                             parameterClass,
                                             parameterType,
+                                            parameterAnns,
                                             message,
                                             values,
                                             ori);
@@ -594,6 +614,7 @@ public final class JAXRSUtils {
     public static Object createHttpParameterValue(Parameter parameter, 
                                             Class<?> parameterClass, 
                                             Type genericParam,
+                                            Annotation[] paramAnns,
                                             Message message,
                                             MultivaluedMap<String, String> values,
                                             OperationResourceInfo ori) {
@@ -607,33 +628,33 @@ public final class JAXRSUtils {
         Object result = null;
         
         if (parameter.getType() == ParameterType.PATH) {
-            result = readFromUriParam(message, parameter.getName(), parameterClass, genericParam, 
-                                      values, defaultValue, !isEncoded);
+            result = readFromUriParam(message, parameter.getName(), parameterClass, genericParam,
+                                      paramAnns, values, defaultValue, !isEncoded);
         } 
         
         if (parameter.getType() == ParameterType.QUERY) {
-            result = readQueryString(parameter.getName(), parameterClass, genericParam, message, 
-                                   defaultValue, !isEncoded);
+            result = readQueryString(parameter.getName(), parameterClass, genericParam, 
+                                     paramAnns, message, defaultValue, !isEncoded);
         }
         
         if (parameter.getType() == ParameterType.MATRIX) {
-            result = processMatrixParam(message, parameter.getName(), parameterClass, genericParam, 
-                                      defaultValue, !isEncoded);
+            result = processMatrixParam(message, parameter.getName(), parameterClass, genericParam,
+                                        paramAnns, defaultValue, !isEncoded);
         }
         
         if (parameter.getType() == ParameterType.FORM) {
             result = processFormParam(message, parameter.getName(), parameterClass, genericParam, 
-                                      defaultValue, !isEncoded);
+                                      paramAnns, defaultValue, !isEncoded);
         }
         
         if (parameter.getType() == ParameterType.COOKIE) {
             result = processCookieParam(message, parameter.getName(), parameterClass, genericParam,
-                                        defaultValue);
+                                        paramAnns, defaultValue);
         } 
         
         if (parameter.getType() == ParameterType.HEADER) {
-            result = processHeaderParam(message, parameter.getName(), parameterClass, genericParam, 
-                                        defaultValue);
+            result = processHeaderParam(message, parameter.getName(), parameterClass, genericParam,
+                                        paramAnns, defaultValue);
         } 
 
         return result;
@@ -641,6 +662,7 @@ public final class JAXRSUtils {
     
     private static Object processMatrixParam(Message m, String key, 
                                              Class<?> pClass, Type genericType,
+                                             Annotation[] paramAnns,
                                              String defaultValue,
                                              boolean decode) {
         List<PathSegment> segments = JAXRSUtils.getPathSegments(
@@ -657,12 +679,13 @@ public final class JAXRSUtils {
             }
             
             if ("".equals(key)) {
-                return InjectionUtils.handleBean(pClass, params, ParameterType.MATRIX, m, false);
+                return InjectionUtils.handleBean(pClass, paramAnns, params, ParameterType.MATRIX, m, false);
             } else {
                 List<String> values = params.get(key);
                 return InjectionUtils.createParameterObject(values, 
                                                             pClass, 
                                                             genericType,
+                                                            paramAnns, 
                                                             defaultValue,
                                                             false,
                                                             ParameterType.MATRIX,
@@ -675,6 +698,7 @@ public final class JAXRSUtils {
     
     private static Object processFormParam(Message m, String key, 
                                            Class<?> pClass, Type genericType,
+                                           Annotation[] paramAnns,
                                            String defaultValue,
                                            boolean decode) {
         
@@ -682,18 +706,15 @@ public final class JAXRSUtils {
         MediaType mt = mc.getHttpHeaders().getMediaType();
         
         @SuppressWarnings("unchecked")
-        MultivaluedMap<String, String> params = (MultivaluedMap<String, String>)m.get(FORM_PARAM_MAP); 
+        MultivaluedMap<String, String> params = 
+            (MultivaluedMap<String, String>)m.get(FormUtils.FORM_PARAM_MAP); 
         
         if (params == null) {
             params = new MetadataMap<String, String>();
-            m.put(FORM_PARAM_MAP, params);
+            m.put(FormUtils.FORM_PARAM_MAP, params);
         
             if (mt == null || mt.isCompatible(MediaType.APPLICATION_FORM_URLENCODED_TYPE)) {
-                String body = (String)m.get("org.apache.cxf.jaxrs.provider.form.body");
-                if (body == null) {
-                    body = FormUtils.readBody(m.getContent(InputStream.class), mt);
-                    m.put("org.apache.cxf.jaxrs.provider.form.body", body);
-                }
+                String body = FormUtils.readBody(m.getContent(InputStream.class), mt);
                 HttpServletRequest request = (HttpServletRequest)m.get(AbstractHTTPDestination.HTTP_REQUEST);
                 FormUtils.populateMapFromString(params, (String)body, decode, request);
             } else {
@@ -713,13 +734,14 @@ public final class JAXRSUtils {
         }
         
         if ("".equals(key)) {
-            return InjectionUtils.handleBean(pClass, params, ParameterType.FORM, m, false);
+            return InjectionUtils.handleBean(pClass, paramAnns, params, ParameterType.FORM, m, false);
         } else {
             List<String> results = params.get(key);
     
             return InjectionUtils.createParameterObject(results, 
                                                         pClass, 
                                                         genericType,
+                                                        paramAnns,
                                                         defaultValue,
                                                         false,
                                                         ParameterType.FORM,
@@ -737,8 +759,9 @@ public final class JAXRSUtils {
     
     private static Object processHeaderParam(Message m, 
                                              String header, 
-                                             Class<?> pClass, 
+                                             Class<?> pClass,
                                              Type genericType, 
+                                             Annotation[] paramAnns,
                                              String defaultValue) {
         
         List<String> values = new HttpHeadersImpl(m).getRequestHeader(header);
@@ -748,6 +771,7 @@ public final class JAXRSUtils {
         return InjectionUtils.createParameterObject(values, 
                                                     pClass, 
                                                     genericType,
+                                                    paramAnns,
                                                     defaultValue,
                                                     false,
                                                     ParameterType.HEADER,
@@ -757,7 +781,8 @@ public final class JAXRSUtils {
     }
 
     private static Object processCookieParam(Message m, String cookieName, 
-                              Class<?> pClass, Type genericType, String defaultValue) {
+                              Class<?> pClass, Type genericType, 
+                              Annotation[] paramAnns, String defaultValue) {
         Cookie c = new HttpHeadersImpl(m).getCookies().get(cookieName);
         
         if (c == null && defaultValue != null) {
@@ -771,7 +796,8 @@ public final class JAXRSUtils {
             return c;
         }
         
-        return InjectionUtils.handleParameter(c.getValue(), pClass, ParameterType.COOKIE, m);
+        return InjectionUtils.handleParameter(c.getValue(), false, pClass, paramAnns, 
+                                              ParameterType.COOKIE, m);
     }
     
     public static <T> T createContextValue(Message m, Type genericType, Class<T> clazz) {
@@ -789,16 +815,24 @@ public final class JAXRSUtils {
         } else if (Request.class.isAssignableFrom(clazz)) {
             o = new RequestImpl(contextMessage);
         } else if (SecurityContext.class.isAssignableFrom(clazz)) {
-            o = new SecurityContextImpl(contextMessage);
+            SecurityContext customContext = contextMessage.get(SecurityContext.class);
+            o = customContext == null ? new SecurityContextImpl(contextMessage) : customContext;
         } else if (Providers.class.isAssignableFrom(clazz)) {
             o = new ProvidersImpl(contextMessage);
         } else if (ContextResolver.class.isAssignableFrom(clazz)) {
             o = createContextResolver(genericType, contextMessage);
         } else if (MessageContext.class.isAssignableFrom(clazz)) {
             o = new MessageContextImpl(m);
+        } else if (SearchContext.class.isAssignableFrom(clazz)) {
+            o = new SearchContextImpl(m);
+        } else if (Application.class.isAssignableFrom(clazz)) {
+            ProviderInfo<?> providerInfo = 
+                (ProviderInfo)contextMessage.getExchange().getEndpoint().get(Application.class.getName());
+            o = providerInfo == null ? providerInfo : providerInfo.getProvider();
+        }
+        if (o == null && contextMessage != null && !MessageUtils.isRequestor(contextMessage)) {
+            o = createServletResourceValue(contextMessage, clazz);
         }
-        
-        o = o == null ? createServletResourceValue(contextMessage, clazz) : o;
         return clazz.cast(o);
     }
     
@@ -844,8 +878,8 @@ public final class JAXRSUtils {
             value = m.get(AbstractHTTPDestination.HTTP_REQUEST);
         }
         if (HttpServletResponse.class.isAssignableFrom(clazz)) {
-            value = new HttpServletResponseFilter(
-                            (HttpServletResponse)m.get(AbstractHTTPDestination.HTTP_RESPONSE), m);
+            HttpServletResponse response = (HttpServletResponse)m.get(AbstractHTTPDestination.HTTP_RESPONSE);
+            value = response != null ? new HttpServletResponseFilter(response, m) : null;
         }
         if (ServletContext.class.isAssignableFrom(clazz)) {
             value = m.get(AbstractHTTPDestination.HTTP_CONTEXT);
@@ -856,22 +890,24 @@ public final class JAXRSUtils {
         
         return clazz.cast(value);
     }
-
+    //CHECKSTYLE:OFF
     private static Object readFromUriParam(Message m,
                                            String parameterName,
                                            Class<?> paramType,
                                            Type genericType,
+                                           Annotation[] paramAnns,
                                            MultivaluedMap<String, String> values,
                                            String defaultValue,
                                            boolean decoded) {
-        
+    //CHECKSTYLE:ON    
         if ("".equals(parameterName)) {
-            return InjectionUtils.handleBean(paramType, values, ParameterType.PATH, m, decoded);
+            return InjectionUtils.handleBean(paramType, paramAnns, values, ParameterType.PATH, m, decoded);
         } else {
             List<String> results = values.get(parameterName);
             return InjectionUtils.createParameterObject(results, 
                                                     paramType, 
                                                     genericType,
+                                                    paramAnns,
                                                     defaultValue,
                                                     decoded,
                                                     ParameterType.PATH,
@@ -885,6 +921,7 @@ public final class JAXRSUtils {
     private static Object readQueryString(String queryName,
                                           Class<?> paramType,
                                           Type genericType,
+                                          Annotation[] paramAnns,
                                           Message m, 
                                           String defaultValue,
                                           boolean decode) {
@@ -892,11 +929,12 @@ public final class JAXRSUtils {
         MultivaluedMap<String, String> queryMap = new UriInfoImpl(m, null).getQueryParameters(decode);
         
         if ("".equals(queryName)) {
-            return InjectionUtils.handleBean(paramType, queryMap, ParameterType.QUERY, m, false);
+            return InjectionUtils.handleBean(paramType, paramAnns, queryMap, ParameterType.QUERY, m, false);
         } else {
             return InjectionUtils.createParameterObject(queryMap.get(queryName), 
                                                         paramType, 
                                                         genericType,
+                                                        paramAnns,
                                                         defaultValue,
                                                         false,
                                                         ParameterType.QUERY, m);
@@ -930,17 +968,21 @@ public final class JAXRSUtils {
         if (!StringUtils.isEmpty(query)) {            
             List<String> parts = Arrays.asList(query.split(sep));
             for (String part : parts) {
-                String[] values = part.split("=");
+                int index = part.indexOf('=');
+                String name = null;
                 String value = null;
-                if (values.length == 1) {
+                if (index == -1) {
+                    name = part;
                     value = "";
-                } else if (decode) {
-                    value = (";".equals(sep) || !decodePlus)
-                        ? HttpUtils.pathDecode(values[1]) : HttpUtils.urlDecode(values[1]); 
                 } else {
-                    value = values[1];
+                    name = part.substring(0, index);
+                    value =  index < part.length() ? part.substring(index + 1) : "";
+                    if (decode || (decodePlus && value.contains("+"))) {
+                        value = (";".equals(sep))
+                            ? HttpUtils.pathDecode(value) : HttpUtils.urlDecode(value); 
+                    }
                 }
-                queries.add(HttpUtils.urlDecode(values[0]), value);
+                queries.add(HttpUtils.urlDecode(name), value);
             }
         }
     }
@@ -1059,11 +1101,14 @@ public final class JAXRSUtils {
                     requiredType.isCompatible(userType) || userType.isCompatible(requiredType);
                 if (!isCompatible && requiredType.getType().equalsIgnoreCase(userType.getType())) {
                     // check if we have composite subtypes
-                    String[] subtypes1 = requiredType.getSubtype().split("\\+");
-                    String[] subtypes2 = userType.getSubtype().split("\\+");
-                    if (subtypes1.length == 2 && subtypes2.length == 2
-                        && subtypes1[1].equalsIgnoreCase(subtypes2[1])
-                        && (subtypes1[0].equals("*") || subtypes2[0].equals("*"))) {
+                    String subType1 = requiredType.getSubtype();
+                    String subTypeAfterPlus1 = splitMediaSubType(subType1); 
+                    String subType2 = userType.getSubtype();
+                    String subTypeAfterPlus2 = splitMediaSubType(subType2);
+                    
+                    if (subTypeAfterPlus1 != null && subTypeAfterPlus2 != null
+                        && subTypeAfterPlus1.equalsIgnoreCase(subTypeAfterPlus2)
+                        && (subType1.charAt(0) == '*' || subType2.charAt(0) == '*')) {
                         isCompatible = true;
                     }
                 }
@@ -1079,10 +1124,10 @@ public final class JAXRSUtils {
                     if (!parametersMatched) {
                         continue;
                     }
-                    
+                   
                     String type = requiredType.getType().equals(MediaType.MEDIA_TYPE_WILDCARD) 
                                       ? userType.getType() : requiredType.getType();
-                    String subtype = requiredType.getSubtype().equals(MediaType.MEDIA_TYPE_WILDCARD) 
+                    String subtype = requiredType.getSubtype().startsWith(MediaType.MEDIA_TYPE_WILDCARD) 
                                       ? userType.getSubtype() : requiredType.getSubtype();
                     Map<String, String> parameters = userType.getParameters();
                     if (addRequiredParamsIfPossible) {
@@ -1102,6 +1147,11 @@ public final class JAXRSUtils {
         
     }
     
+    static String splitMediaSubType(String type) {
+        int index = type.indexOf('+');
+        return index == -1 ? null : type.substring(index + 1);
+    }
+    
     public static List<MediaType> intersectMimeTypes(List<MediaType> mimeTypesA, 
                                                      MediaType mimeTypeB) {
         return intersectMimeTypes(mimeTypesA, 
@@ -1165,7 +1215,7 @@ public final class JAXRSUtils {
         
     public static boolean propogateException(Message m) {
         
-        Object value = m.getContextualProperty(PROPOGATE_EXCEPTION);
+        Object value = m.getContextualProperty(PROPAGATE_EXCEPTION);
         
         if (value == null) {
             return true;
