@@ -27,20 +27,28 @@ import java.lang.reflect.Method;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashMap;
+import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
+import java.util.TreeSet;
 
 import javax.ws.rs.core.MediaType;
 import javax.ws.rs.ext.MessageBodyReader;
 import javax.xml.bind.JAXBContext;
 import javax.xml.bind.JAXBElement;
 import javax.xml.bind.Unmarshaller;
+import javax.xml.bind.annotation.XmlAccessType;
+import javax.xml.bind.annotation.XmlAccessorType;
 import javax.xml.bind.annotation.XmlAnyElement;
+import javax.xml.bind.annotation.XmlElement;
+import javax.xml.bind.annotation.XmlElementWrapper;
 import javax.xml.bind.annotation.XmlMixed;
 import javax.xml.bind.annotation.XmlRootElement;
 import javax.xml.bind.annotation.XmlType;
 import javax.xml.transform.stream.StreamSource;
 
+import org.apache.cxf.jaxrs.fortest.jaxb.packageinfo.Book2;
 import org.apache.cxf.jaxrs.impl.MetadataMap;
 import org.apache.cxf.jaxrs.provider.JAXBElementProviderTest.TagVO2Holder;
 import org.apache.cxf.jaxrs.resources.Book;
@@ -57,6 +65,65 @@ import org.junit.Test;
 
 public class JSONProviderTest extends Assert {
 
+    @SuppressWarnings("unchecked")
+    @Test
+    public void testReadEmbeddedArray() throws Exception {
+        String input = 
+                "{\"store\":"
+               + "{\"books\":{"
+               + "     \"book\":["
+               + "         {           "
+               + "           \"name\":\"CXF 1\""
+               + "         },          "
+               + "         {           "
+               + "           \"name\":\"CXF 2\""
+               + "         }           "
+               + "      ]              " 
+               + "   }                 "
+               + " }                   "
+               + "}                    ";
+        
+        Object storeObject = new JSONProvider().readFrom((Class)Store.class, null, null, 
+                                       null, null, new ByteArrayInputStream(input.getBytes()));
+        Store store = (Store)storeObject;
+        List<Book> books = store.getBooks();
+        assertEquals(2, books.size());
+        assertEquals("CXF 1", books.get(0).getName());
+        assertEquals("CXF 2", books.get(1).getName());
+    }
+    
+    @SuppressWarnings("unchecked")
+    @Test
+    public void testReadEmbeddedArrayWithNamespaces() throws Exception {
+        String input = 
+                "{\"ns1.store\":"
+               + "{\"ns1.books\":{"
+               + "     \"ns1.thebook2\":["
+               + "         {           "
+               + "           \"name\":\"CXF 1\""
+               + "         },          "
+               + "         {           "
+               + "           \"name\":\"CXF 2\""
+               + "         }           "
+               + "      ]              " 
+               + "   }                 "
+               + " }                   "
+               + "}                    ";
+        
+        JSONProvider p = new JSONProvider();
+        Map<String, String> namespaceMap = new HashMap<String, String>();
+        namespaceMap.put("http://superbooks", "ns1");
+        p.setNamespaceMap(namespaceMap);
+        
+        Object storeObject = p.readFrom((Class)QualifiedStore.class, null, null, 
+                                       null, null, new ByteArrayInputStream(input.getBytes()));
+        QualifiedStore store = (QualifiedStore)storeObject;
+        List<Book2> books = store.getBooks();
+        assertEquals(2, books.size());
+        assertEquals("CXF 1", books.get(0).getName());
+        assertEquals("CXF 2", books.get(1).getName());
+    }
+    
     
     @Test
     public void testWriteCollectionWithoutXmlRootElement() 
@@ -197,6 +264,22 @@ public class JSONProviderTest extends Assert {
         
     }
     
+    @Test
+    public void testWriteBookWithStringConverter() throws Exception {
+        JSONProvider p = new JSONProvider();
+        p.setConvertTypesToStrings(true);
+        Book book = new Book("CXF", 125);
+        
+        ByteArrayOutputStream os = new ByteArrayOutputStream();
+        
+        p.writeTo(book, (Class)Book.class, Book.class, Book.class.getAnnotations(), 
+                  MediaType.APPLICATION_JSON_TYPE, new MetadataMap<String, Object>(), os);
+        
+        String s = os.toString();
+        assertEquals("{\"Book\":{\"id\":\"125\",\"name\":\"CXF\",\"state\":\"\"}}", s);
+        
+    }
+    
     @Test
     public void testWriteToSingleTagBadgerFish() throws Exception {
         JSONProvider p = new JSONProvider();
@@ -308,17 +391,72 @@ public class JSONProviderTest extends Assert {
     }
     
     @Test
-    @Ignore
+    public void testWriteUnqualifiedCollection() throws Exception {
+        JSONProvider p = new JSONProvider();
+        List<Book> books = new ArrayList<Book>();
+        books.add(new Book("CXF", 123L));
+        ByteArrayOutputStream os = new ByteArrayOutputStream();
+        Method m = CollectionsResource.class.getMethod("getBooks", new Class[0]);
+        p.writeTo(books, m.getReturnType(), m.getGenericReturnType(), new Annotation[0], 
+                  MediaType.APPLICATION_JSON_TYPE, new MetadataMap<String, Object>(), os);
+        assertEquals("{\"Book\":[{\"id\":123,\"name\":\"CXF\",\"state\":\"\"}]}",
+                     os.toString());
+        
+    }
+    
+    @Test
+    public void testReadUnqualifiedCollection() throws Exception {
+        String data = "{\"Book\":[{\"id\":\"123\",\"name\":\"CXF in Action\"}"
+            + ",{\"id\":\"124\",\"name\":\"CXF Rocks\"}]}";
+        doReadUnqualifiedCollection(data, "setBooks", List.class);
+    }
+    
+    @SuppressWarnings("unchecked")
+    private void doReadUnqualifiedCollection(String data, String mName, Class<?> type) throws Exception {
+        JSONProvider provider = new JSONProvider();
+        Method m = CollectionsResource.class.getMethod(mName, 
+                                                       new Class[]{type});
+        ByteArrayInputStream is = new ByteArrayInputStream(data.getBytes());
+        Object o = provider.readFrom(
+                      (Class)m.getParameterTypes()[0], m.getGenericParameterTypes()[0],
+                       new Annotation[0], MediaType.APPLICATION_JSON_TYPE, 
+                       new MetadataMap<String, String>(), is);
+        assertNotNull(o);
+        Book b1 = null;
+        Book b2 = null;
+        if (type.isArray()) {
+            assertEquals(2, ((Book[])o).length);
+            b1 = ((Book[])o)[0];
+            b2 = ((Book[])o)[1];
+        } else if (type == Set.class) {
+            Set<Book> set = (Set)o;
+            List<Book> books = new ArrayList<Book>(new TreeSet<Book>(set));
+            b1 = books.get(0);
+            b2 = books.get(1);
+        } else {
+            List<Book> books = (List<Book>)o;
+            b1 = books.get(0);
+            b2 = books.get(1);
+        }
+        
+        assertEquals(123, b1.getId());
+        assertEquals("CXF in Action", b1.getName());
+        
+        assertEquals(124, b2.getId());
+        assertEquals("CXF Rocks", b2.getName());    
+    }
+    
+    @Test
     public void testReadQualifiedCollection() throws Exception {
-        String data = "{\"ns1.tag\":[{\"group\":\"b\",\"name\":\"a\"}"
+        String data = "{\"ns1.thetag\":[{\"group\":\"b\",\"name\":\"a\"}"
             + ",{\"group\":\"d\",\"name\":\"c\"}]}";
         doReadQualifiedCollection(data, false);
     }
     
     @Test
-    @Ignore
+    @Ignore("Enable once http://jira.codehaus.org/browse/JETTISON-104 gets resolved")
     public void testReadQualifiedArray() throws Exception {
-        String data = "{\"ns1.tag\":[{\"group\":\"b\",\"name\":\"a\"}"
+        String data = "{\"ns1.thetag\":[{\"group\":\"b\",\"name\":\"a\"}"
             + ",{\"group\":\"d\",\"name\":\"c\"}]}";
         doReadQualifiedCollection(data, true);
     }
@@ -327,7 +465,6 @@ public class JSONProviderTest extends Assert {
     public void doReadQualifiedCollection(String data, boolean isArray) throws Exception {
         
         JSONProvider provider = new JSONProvider();
-        provider.setCollectionWrapperName("{http://tags}tag");
         Map<String, String> namespaceMap = new HashMap<String, String>();
         namespaceMap.put("http://tags", "ns1");
         provider.setNamespaceMap(namespaceMap);
@@ -356,11 +493,11 @@ public class JSONProviderTest extends Assert {
             t1 = (TagVO2)((Object[])o)[0];
             t2 = (TagVO2)((Object[])o)[1];
         }
-        assertEquals("A", t1.getName());
-        assertEquals("B", t1.getGroup());
+        assertEquals("a", t1.getName());
+        assertEquals("b", t1.getGroup());
         
-        assertEquals("C", t2.getName());
-        assertEquals("D", t2.getGroup());
+        assertEquals("c", t2.getName());
+        assertEquals("d", t2.getGroup());
     }
     
     @Test
@@ -448,6 +585,28 @@ public class JSONProviderTest extends Assert {
         assertEquals(1, list.size());
         SuperBook book = (SuperBook)list.get(0);
         assertEquals(124L, book.getSuperId());
+        assertEquals(123L, book.getId());
+        assertEquals("CXF Rocks", book.getName());
+    }
+    
+    @SuppressWarnings("unchecked")
+    @Test
+    public void testReadListOfDerivedTypesWithNullField() throws Exception {
+        JSONProvider p = new JSONProvider();
+        Map<String, String> namespaceMap = new HashMap<String, String>();
+        namespaceMap.put("http://www.w3.org/2001/XMLSchema-instance", "xsins");
+        p.setNamespaceMap(namespaceMap);
+        String data = "{\"books2\":{\"books\":{\"@xsins.type\":\"superBook\",\"id\":123,"
+            + "\"name\":null,\"superId\":124}}}";
+        byte[] bytes = data.getBytes();
+        Object books2Object = p.readFrom((Class)Books2.class, null, null, 
+                                          null, null, new ByteArrayInputStream(bytes));
+        Books2 books = (Books2)books2Object;
+        List<? extends Book> list = books.getBooks();
+        assertEquals(1, list.size());
+        SuperBook book = (SuperBook)list.get(0);
+        assertEquals(124L, book.getSuperId());
+        assertEquals(0, book.getName().length());
     }
     
     @Test
@@ -741,10 +900,6 @@ public class JSONProviderTest extends Assert {
         Map<String, String> map = new HashMap<String, String>();
         map.put("tagVO", "{http://tagsvo2}t");
         provider.setOutAppendElements(map);
-        Map<String, String> nsmap = new HashMap<String, String>();
-        nsmap.put("http://tagsvo2", "ps1");
-        provider.setNamespaceMap(nsmap);
-        
         TagVO tag = new TagVO("A", "B");
         ByteArrayOutputStream bos = new ByteArrayOutputStream();
         provider.writeTo(tag, TagVO.class, TagVO.class,
@@ -787,14 +942,11 @@ public class JSONProviderTest extends Assert {
         Map<String, String> map = new HashMap<String, String>();
         map.put("{http://tags}thetag", "{http://tagsvo2}t");
         provider.setOutAppendElements(map);
-        Map<String, String> nsmap = new HashMap<String, String>();
-        nsmap.put("http://tagsvo2", "ps1");
-        provider.setNamespaceMap(nsmap);
         TagVO2 tag = new TagVO2("A", "B");
         ByteArrayOutputStream bos = new ByteArrayOutputStream();
         provider.writeTo(tag, TagVO2.class, TagVO2.class,
                        new Annotation[0], MediaType.TEXT_XML_TYPE, new MetadataMap<String, Object>(), bos);
-        String expected = "{\"ps1.t\":{\"ns2.thetag\":{\"group\":\"B\",\"name\":\"A\"}}}";
+        String expected = "{\"ps1.t\":{\"ns1.thetag\":{\"group\":\"B\",\"name\":\"A\"}}}";
         assertEquals(expected, bos.toString());
     }
     
@@ -803,9 +955,6 @@ public class JSONProviderTest extends Assert {
         JSONProvider provider = new JSONProvider();
         Map<String, String> map = new HashMap<String, String>();
         map.put("{http://tags}thetag", "{http://tagsvo2}t");
-        Map<String, String> nsmap = new HashMap<String, String>();
-        nsmap.put("http://tagsvo2", "ns2");
-        provider.setNamespaceMap(nsmap);
         provider.setOutTransformElements(map);
         TagVO2 tag = new TagVO2("A", "B");
         ByteArrayOutputStream bos = new ByteArrayOutputStream();
@@ -838,9 +987,6 @@ public class JSONProviderTest extends Assert {
         Map<String, String> map = new HashMap<String, String>();
         map.put("tagVO", "{http://tags}thetag");
         provider.setOutTransformElements(map);
-        Map<String, String> nsmap = new HashMap<String, String>();
-        nsmap.put("http://tags", "ps1");
-        provider.setNamespaceMap(nsmap);
         TagVO tag = new TagVO("A", "B");
         ByteArrayOutputStream bos = new ByteArrayOutputStream();
         provider.writeTo(tag, TagVO.class, TagVO.class,
@@ -923,4 +1069,27 @@ public class JSONProviderTest extends Assert {
         }
     }
 
+    @XmlAccessorType(XmlAccessType.PUBLIC_MEMBER)
+    @XmlRootElement(name = "store")
+    public static class Store {
+        private List<Book> books = new LinkedList<Book>();
+       
+        @XmlElement(name = "book")
+        @XmlElementWrapper(name = "books")
+        public List<Book> getBooks() {
+            return books;
+        }
+    }
+    
+    @XmlAccessorType(XmlAccessType.PUBLIC_MEMBER)
+    @XmlRootElement(name = "store", namespace = "http://superbooks")
+    public static class QualifiedStore {
+        private List<Book2> books = new LinkedList<Book2>();
+       
+        @XmlElement(name = "thebook2", namespace = "http://superbooks")
+        @XmlElementWrapper(name = "books", namespace = "http://superbooks")
+        public List<Book2> getBooks() {
+            return books;
+        }
+    }
 }
