@@ -33,6 +33,7 @@ import javax.servlet.ServletContext;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 import javax.ws.rs.WebApplicationException;
+import javax.ws.rs.core.Application;
 import javax.ws.rs.core.Cookie;
 import javax.ws.rs.core.HttpHeaders;
 import javax.ws.rs.core.MediaType;
@@ -46,11 +47,15 @@ import javax.ws.rs.ext.Providers;
 import javax.xml.bind.JAXBContext;
 
 import org.apache.cxf.endpoint.Endpoint;
+import org.apache.cxf.endpoint.Server;
 import org.apache.cxf.jaxrs.Customer;
 import org.apache.cxf.jaxrs.Customer2;
+import org.apache.cxf.jaxrs.CustomerApplication;
 import org.apache.cxf.jaxrs.CustomerGender;
 import org.apache.cxf.jaxrs.CustomerParameterHandler;
 import org.apache.cxf.jaxrs.JAXBContextProvider;
+import org.apache.cxf.jaxrs.JAXRSInvoker;
+import org.apache.cxf.jaxrs.JAXRSServerFactoryBean;
 import org.apache.cxf.jaxrs.JAXRSServiceFactoryBean;
 import org.apache.cxf.jaxrs.JAXRSServiceImpl;
 import org.apache.cxf.jaxrs.SimpleFactory;
@@ -73,6 +78,7 @@ import org.apache.cxf.jaxrs.model.ClassResourceInfo;
 import org.apache.cxf.jaxrs.model.MethodDispatcher;
 import org.apache.cxf.jaxrs.model.OperationResourceInfo;
 import org.apache.cxf.jaxrs.model.URITemplate;
+import org.apache.cxf.jaxrs.provider.FormEncodingProvider;
 import org.apache.cxf.jaxrs.provider.ProviderFactory;
 import org.apache.cxf.message.Exchange;
 import org.apache.cxf.message.ExchangeImpl;
@@ -111,6 +117,65 @@ public class JAXRSUtilsTest extends Assert {
                      org.apache.cxf.jaxrs.resources.BookStoreNoSubResource.class);
     }
     
+    @Test
+    public void testInjectApplicationInSingleton() throws Exception {
+        CustomerApplication app = new CustomerApplication();
+        JAXRSServerFactoryBean sf = new JAXRSServerFactoryBean();
+        Customer customer = new Customer();
+        sf.setServiceBeanObjects(customer);
+        sf.setApplication(app);
+        sf.setStart(false);
+        Server server = sf.create();  
+        assertSame(app, customer.getApplication1());
+        assertSame(app, customer.getApplication2());
+        ThreadLocalProxy proxy = (ThreadLocalProxy)app.getUriInfo();
+        assertNotNull(proxy);
+        invokeCustomerMethod(sf.getServiceFactory().getClassResourceInfo().get(0),
+                             customer, server);
+        assertSame(app, customer.getApplication2());
+        assertTrue(proxy.get() instanceof UriInfo);
+    }
+    
+    @Test
+    public void testInjectApplicationInPerRequestResource() throws Exception {
+        CustomerApplication app = new CustomerApplication();
+        JAXRSServerFactoryBean sf = new JAXRSServerFactoryBean();
+        sf.setServiceClass(Customer.class);
+        sf.setApplication(app);
+        sf.setStart(false);
+        Server server = sf.create();  
+        
+        ThreadLocalProxy proxy = (ThreadLocalProxy)app.getUriInfo();
+        assertNotNull(proxy);
+        
+        ClassResourceInfo cri = sf.getServiceFactory().getClassResourceInfo().get(0);
+        
+        Customer customer = (Customer)cri.getResourceProvider().getInstance(
+             new MessageImpl());
+        
+        assertNull(customer.getApplication1());
+        assertNull(customer.getApplication2());
+        
+        invokeCustomerMethod(cri, customer, server);
+        assertSame(app, customer.getApplication1());
+        assertSame(app, customer.getApplication2());
+        
+        assertTrue(proxy.get() instanceof UriInfo);
+    }
+    
+    private void invokeCustomerMethod(ClassResourceInfo cri, 
+        Customer customer, Server server) throws Exception {
+        OperationResourceInfo ori = cri.getMethodDispatcher().getOperationResourceInfo(
+            Customer.class.getMethod("test", new Class[]{}));
+        JAXRSInvoker invoker = new JAXRSInvoker();
+        Exchange exc = new ExchangeImpl();
+        exc.put(Endpoint.class, server.getEndpoint());
+        Message inMessage = new MessageImpl();
+        exc.setInMessage(inMessage);
+        exc.put(OperationResourceInfo.class, ori);
+        invoker.invoke(exc, Collections.emptyList(), customer);
+    }
+    
     @Test
     public void testSelectBetweenMultipleResourceClasses2() throws Exception {
         JAXRSServiceFactoryBean sf = new JAXRSServiceFactoryBean();
@@ -305,6 +370,33 @@ public class JAXRSUtilsTest extends Assert {
         assertEquals("application/bar+xml", candidateList.get(0).toString());
     }
     
+    @Test
+    public void testIntersectMimeTypesCompositeSubtype2() throws Exception {
+        List <MediaType> candidateList = 
+            JAXRSUtils.intersectMimeTypes("application/bar+xml", "application/bar+xml");  
+
+        assertEquals(1, candidateList.size());
+        assertEquals("application/bar+xml", candidateList.get(0).toString());
+    }
+    
+    @Test
+    public void testIntersectMimeTypesCompositeSubtype3() throws Exception {
+        List <MediaType> candidateList = 
+            JAXRSUtils.intersectMimeTypes("application/*+xml", "application/bar+xml");  
+
+        assertEquals(1, candidateList.size());
+        assertEquals("application/bar+xml", candidateList.get(0).toString());
+    }
+    
+    @Test
+    public void testIntersectMimeTypesCompositeSubtype4() throws Exception {
+        List <MediaType> candidateList = 
+            JAXRSUtils.intersectMimeTypes("application/*+xml", "application/bar+json");  
+
+        assertEquals(0, candidateList.size());
+        
+    }
+    
     @Test
     public void testIntersectMimeTypes() throws Exception {
         //test basic
@@ -761,6 +853,37 @@ public class JAXRSUtilsTest extends Assert {
         verifyParametersBean(m, null, messageImpl, null, complexMessageImpl);
     }
     
+    @Test
+    public void testXmlAdapterBean() throws Exception {
+        Class[] argType = {Customer.CustomerBean.class};
+        Method m = Customer.class.getMethod("testXmlAdapter", argType);
+        Message messageImpl = createMessage();
+        messageImpl.put(Message.QUERY_STRING, "a=aValue");
+
+        List<Object> params = JAXRSUtils.processParameters(new OperationResourceInfo(m, null), 
+                                                           null, messageImpl);
+        assertEquals(1, params.size());
+        
+        Customer.CustomerBean bean = (Customer.CustomerBean)params.get(0);
+        assertEquals("aValue", bean.getA());
+    }
+    
+    @Test
+    public void testXmlAdapterBean2() throws Exception {
+        Class[] argType = {Customer.CustomerBean.class};
+        Method m = Customer.class.getMethod("testXmlAdapter2", argType);
+        Message messageImpl = createMessage();
+        messageImpl.put(Message.QUERY_STRING, "a=aValue");
+
+        List<Object> params = JAXRSUtils.processParameters(new OperationResourceInfo(m, null), 
+                                                           null, messageImpl);
+        assertEquals(1, params.size());
+        
+        Customer.CustomerBean bean = (Customer.CustomerBean)params.get(0);
+        assertEquals("aValue", bean.getA());
+    }
+    
+    
     @Test
     public void testPathParametersBean() throws Exception {
         Class[] argType = {Customer.CustomerBean.class};
@@ -975,14 +1098,14 @@ public class JAXRSUtilsTest extends Assert {
     @SuppressWarnings("unchecked")
     @Test
     public void testMatrixParameters() throws Exception {
-        Class[] argType = {String.class, String.class, String.class, String.class, List.class};
+        Class[] argType = {String.class, String.class, String.class, String.class, List.class, String.class};
         Method m = Customer.class.getMethod("testMatrixParam", argType);
         MessageImpl messageImpl = new MessageImpl();
         
-        messageImpl.put(Message.REQUEST_URI, "/foo;p4=0;p3=3/bar;p1=1;p2/baz;p4=4;p4=5");
+        messageImpl.put(Message.REQUEST_URI, "/foo;p4=0;p3=3/bar;p1=1;p2/baz;p4=4;p4=5;p5");
         List<Object> params = JAXRSUtils.processParameters(new OperationResourceInfo(m, null), 
                                                            null, messageImpl);
-        assertEquals("5 Matrix params should've been identified", 5, params.size());
+        assertEquals("5 Matrix params should've been identified", 6, params.size());
         
         assertEquals("First Matrix Parameter not matched correctly", 
                      "1", params.get(0));
@@ -997,6 +1120,8 @@ public class JAXRSUtilsTest extends Assert {
         assertEquals("0", list.get(0));
         assertEquals("4", list.get(1));
         assertEquals("5", list.get(2));
+        assertEquals("Sixth Matrix Parameter was not matched correctly", 
+                     "", params.get(5));
     }
     
     @Test
@@ -1054,6 +1179,47 @@ public class JAXRSUtilsTest extends Assert {
         assertEquals("3", list.get(1));
     }
     
+    @SuppressWarnings("unchecked")
+    @Test
+    public void testFormParametersAndMap() throws Exception {
+        Class[] argType = {MultivaluedMap.class, String.class, List.class};
+        Method m = Customer.class.getMethod("testMultivaluedMapAndFormParam", argType);
+        final Message messageImpl = createMessage();
+        String body = "p1=1&p2=2&p2=3";
+        messageImpl.put(Message.REQUEST_URI, "/foo");
+        messageImpl.put("Content-Type", MediaType.APPLICATION_FORM_URLENCODED);
+        messageImpl.setContent(InputStream.class, new ByteArrayInputStream(body.getBytes()));
+        
+        ProviderFactory.getInstance(messageImpl).registerUserProvider(
+            new FormEncodingProvider() {
+                @Override
+                protected void persistParamsOnMessage(MultivaluedMap<String, String> params) {
+                    messageImpl.put(FormUtils.FORM_PARAM_MAP, params);    
+                }
+            });
+        
+        List<Object> params = JAXRSUtils.processParameters(new OperationResourceInfo(m, null), 
+                                                           new MetadataMap<String, String>(), messageImpl);
+        assertEquals("3 params should've been identified", 3, params.size());
+        
+        MultivaluedMap<String, String> map = (MultivaluedMap<String, String>)params.get(0);
+        assertEquals(2, map.size());
+        assertEquals(1, map.get("p1").size());
+        assertEquals("First map parameter not matched correctly", 
+                     "1", map.getFirst("p1"));
+        assertEquals(2, map.get("p2").size());
+        
+        assertEquals("2", map.get("p2").get(0));
+        assertEquals("3", map.get("p2").get(1));
+        
+        assertEquals("First Form Parameter not matched correctly", 
+                     "1", params.get(1));
+        List<String> list = (List<String>)params.get(2);
+        assertEquals(2, list.size());
+        assertEquals("2", list.get(0));
+        assertEquals("3", list.get(1));
+    }
+    
     @Test
     public void testSelectResourceMethod() throws Exception {
         ClassResourceInfo cri = new ClassResourceInfo(Customer.class);
@@ -1298,7 +1464,7 @@ public class JAXRSUtilsTest extends Assert {
         OperationResourceInfo ori = new OperationResourceInfo(Customer.class.getMethods()[0],
                                                               cri); 
         Message message = new MessageImpl();
-        JAXRSUtils.handleSetters(ori, c, message);
+        InjectionUtils.injectContextMethods(c, ori.getClassResourceInfo(), message);
         assertNotNull(c.getUriInfo());
         assertSame(ThreadLocalUriInfo.class, c.getUriInfo().getClass());
         assertSame(UriInfoImpl.class, 
@@ -1314,12 +1480,12 @@ public class JAXRSUtilsTest extends Assert {
         Customer c = new Customer();
         OperationResourceInfo ori = new OperationResourceInfo(Customer.class.getMethods()[0],
                                                               cri);
-        Message m = new MessageImpl();
+        Message m = createMessage();
         MultivaluedMap<String, String> headers = new MetadataMap<String, String>();
         headers.add("AHeader2", "theAHeader2");
         m.put(Message.PROTOCOL_HEADERS, headers);
         m.put(Message.QUERY_STRING, "a=aValue&query2=b");
-        JAXRSUtils.handleSetters(ori, c, m);
+        JAXRSUtils.injectParameters(ori, c, m);
         assertEquals("aValue", c.getQueryParam());
         assertEquals("theAHeader2", c.getAHeader2());
     }
@@ -1331,16 +1497,17 @@ public class JAXRSUtilsTest extends Assert {
         Customer c = new Customer();
         OperationResourceInfo ori = new OperationResourceInfo(Customer.class.getMethods()[0],
                                                               cri);
-        Message m = new MessageImpl();
+        Message m = createMessage();
+        
         MultivaluedMap<String, String> headers = new MetadataMap<String, String>();
         headers.add("AHeader", "theAHeader");
         m.put(Message.PROTOCOL_HEADERS, headers);
         m.put(Message.QUERY_STRING, "b=bValue");
-        JAXRSUtils.handleSetters(ori, c, m);
+        JAXRSUtils.injectParameters(ori, c, m);
         assertEquals("bValue", c.getB());
         assertEquals("theAHeader", c.getAHeader());
     }
-
+    
     @Test
     public void testContextResolverParam() throws Exception {
         
@@ -1483,6 +1650,8 @@ public class JAXRSUtilsTest extends Assert {
         Endpoint endpoint = EasyMock.createMock(Endpoint.class);
         endpoint.getEndpointInfo();
         EasyMock.expectLastCall().andReturn(null).anyTimes();
+        endpoint.get(Application.class.getName());
+        EasyMock.expectLastCall().andReturn(null);
         endpoint.size();
         EasyMock.expectLastCall().andReturn(0).anyTimes();
         endpoint.isEmpty();
