@@ -31,6 +31,7 @@ import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.TreeMap;
 
 import org.apache.cxf.Bus;
 import org.apache.cxf.bus.CXFBusImpl;
@@ -212,7 +213,7 @@ public class HTTPConduitURLEasyMockTest extends Assert {
     }
     
     private void setUpHeaders(Message message) {
-        Map<String, List<String>> headers = new HashMap<String, List<String>>();
+        Map<String, List<String>> headers = new TreeMap<String, List<String>>(String.CASE_INSENSITIVE_ORDER);
         List<String> contentTypes = new ArrayList<String>();
         contentTypes.add("text/xml;charset=utf8");
         headers.put("content-type", contentTypes);
@@ -255,7 +256,7 @@ public class HTTPConduitURLEasyMockTest extends Assert {
         
         if (send) {
             //proxy = control.createMock(Proxy.class);
-            proxy =  null;
+            proxy = null;
             connection =
                 control.createMock(HttpURLConnection.class);
             connectionFactory.createConnection(
