@@ -21,6 +21,7 @@ package org.apache.cxf.ws.security.trust;
 
 import java.io.IOException;
 import java.io.InputStream;
+import java.io.StringReader;
 import java.net.URL;
 import java.security.PublicKey;
 import java.security.cert.X509Certificate;
@@ -50,7 +51,7 @@ import org.apache.cxf.BusException;
 import org.apache.cxf.binding.soap.SoapBindingConstants;
 import org.apache.cxf.binding.soap.model.SoapOperationInfo;
 import org.apache.cxf.common.classloader.ClassLoaderUtils;
-import org.apache.cxf.common.i18n.Message;
+//import org.apache.cxf.common.i18n.Message;
 import org.apache.cxf.common.logging.LogUtils;
 import org.apache.cxf.common.util.ModCountCopyOnWriteArrayList;
 import org.apache.cxf.common.util.StringUtils;
@@ -68,6 +69,7 @@ import org.apache.cxf.helpers.DOMUtils;
 import org.apache.cxf.interceptor.Fault;
 import org.apache.cxf.interceptor.Interceptor;
 import org.apache.cxf.interceptor.InterceptorProvider;
+import org.apache.cxf.message.Message;
 import org.apache.cxf.resource.ResourceManager;
 import org.apache.cxf.service.Service;
 import org.apache.cxf.service.model.BindingInfo;
@@ -110,6 +112,7 @@ import org.apache.ws.security.processor.EncryptedKeyProcessor;
 import org.apache.ws.security.util.Base64;
 import org.apache.ws.security.util.WSSecurityUtil;
 import org.apache.ws.security.util.XmlSchemaDateFormat;
+import org.apache.xml.security.keys.content.X509Data;
 import org.apache.xml.security.keys.content.keyvalues.DSAKeyValue;
 import org.apache.xml.security.keys.content.keyvalues.RSAKeyValue;
 
@@ -118,6 +121,7 @@ import org.apache.xml.security.keys.content.keyvalues.RSAKeyValue;
  */
 public class STSClient implements Configurable, InterceptorProvider {
     private static final Logger LOG = LogUtils.getL7dLogger(STSClient.class);
+    private static final String SAML2_NS = "urn:oasis:names:tc:SAML:2.0:assertion";
     
     Bus bus;
     String name = "default.sts-client";
@@ -136,16 +140,27 @@ public class STSClient implements Configurable, InterceptorProvider {
     AlgorithmSuite algorithmSuite;
     String namespace = STSUtils.WST_NS_05_12;
     String addressingNamespace;
+    Element onBehalfOfElement;
 
+    boolean useCertificateForConfirmationKeyInfo;
     boolean isSecureConv;
     int ttl = 300;
+    
+    Object actAs;
+    String tokenType;
+    String keyType;
+    boolean sendKeyType = true;
 
     Map<String, Object> ctx = new HashMap<String, Object>();
     
-    List<Interceptor> in = new ModCountCopyOnWriteArrayList<Interceptor>();
-    List<Interceptor> out = new ModCountCopyOnWriteArrayList<Interceptor>();
-    List<Interceptor> outFault  = new ModCountCopyOnWriteArrayList<Interceptor>();
-    List<Interceptor> inFault  = new ModCountCopyOnWriteArrayList<Interceptor>();
+    List<Interceptor<? extends Message>> in 
+        = new ModCountCopyOnWriteArrayList<Interceptor<? extends Message>>();
+    List<Interceptor<? extends Message>> out 
+        = new ModCountCopyOnWriteArrayList<Interceptor<? extends Message>>();
+    List<Interceptor<? extends Message>> outFault  
+        = new ModCountCopyOnWriteArrayList<Interceptor<? extends Message>>();
+    List<Interceptor<? extends Message>> inFault 
+        = new ModCountCopyOnWriteArrayList<Interceptor<? extends Message>>();
     List<AbstractFeature> features;
 
     public STSClient(Bus b) {
@@ -164,25 +179,27 @@ public class STSClient implements Configurable, InterceptorProvider {
         this.location = location;
     }
 
-    public void setPolicy(Policy policy) {
-        this.policy = policy;
-        if (algorithmSuite == null) {
-            Iterator i = policy.getAlternatives();
-            while (i.hasNext() && algorithmSuite == null) {
-                List<PolicyComponent> p = CastUtils.cast((List)i.next());
-                for (PolicyComponent p2 : p) {
-                    if (p2 instanceof Binding) {
-                        algorithmSuite = ((Binding)p2).getAlgorithmSuite();
-                    }
-                }
-            }
+    /**
+     * Sets the WS-P policy that is applied to communications between this client and the remote server
+     * if no value is supplied for {@link #setWsdlLocation(String)}.
+     * <p/>
+     * Accepts {@link Policy} or {@link Element} as input.
+     *
+     * @param newPolicy the policy object
+     *
+     * @throws IllegalArgumentException if {@code newPolicy} is not one of the supported types.
+     */
+    public void setPolicy(Object newPolicy) {
+        if (newPolicy instanceof Policy) {
+            this.setPolicyInternal((Policy) newPolicy);
+        } else if (newPolicy instanceof Element) {
+            this.setPolicyInternal((Element) newPolicy);    
+        } else {
+            throw new IllegalArgumentException("Unsupported policy object.  Type must be "
+                       + "org.apache.neethi.Policy or org.w3c.dom.Element.");
         }
     }
 
-    public void setPolicy(Element policy) {
-        setPolicy(bus.getExtension(PolicyBuilder.class).getPolicy(policy));
-    }
-
     public void setSoap12() {
         soapVersion = SoapBindingConstants.SOAP12_BINDING_ID;
     }
@@ -252,6 +269,9 @@ public class STSClient implements Configurable, InterceptorProvider {
     public void setWsdlLocation(String wsdl) {
         wsdlLocation = wsdl;
     }
+    public String getWsdlLocation() {
+        return wsdlLocation;
+    }
 
     public void setServiceName(String qn) {
         serviceName = QName.valueOf(qn);
@@ -264,10 +284,21 @@ public class STSClient implements Configurable, InterceptorProvider {
     public void setServiceQName(QName qn) {
         serviceName = qn;
     }
+    public QName getServiceQName() {
+        return serviceName;
+    }
 
     public void setEndpointQName(QName qn) {
         endpointName = qn;
     }
+    public QName getEndpointQName() {
+        return endpointName;
+    }
+    
+    public void setActAs(Object actAs) {
+        this.actAs = actAs;
+    }
+    
     public void setKeySize(int i) {
         keySize = i;
     }
@@ -275,6 +306,57 @@ public class STSClient implements Configurable, InterceptorProvider {
         return keySize;
     }
 
+    public void setTokenType(String tokenType) {
+        this.tokenType = tokenType;
+    }
+    
+    public void setSendKeyType(boolean sendKeyType) {
+        this.sendKeyType = sendKeyType;
+    }
+    
+    public void setKeyType(String keyType) {
+        this.keyType = keyType;
+    }
+    
+    public void setOnBehalfOfElement(Element onBehalfOfElement) {
+        this.onBehalfOfElement = onBehalfOfElement;
+    }
+
+    /**
+     * Indicate whether to use the signer's public X509 certificate for the subject confirmation key info 
+     * when creating a RequestsSecurityToken message. If the property is set to 'false', only the public key 
+     * value will be provided in the request. If the property is set to 'true' the complete certificate will 
+     * be sent in the request.
+     * 
+     * Note: this setting is only applicable for assertions that use an asymmetric proof key
+     */
+    public void setUseCertificateForConfirmationKeyInfo(boolean useCertificate) {
+        this.useCertificateForConfirmationKeyInfo = useCertificate;
+    }
+    
+    public boolean isUseCertificateForConfirmationKeyInfo() {
+        return useCertificateForConfirmationKeyInfo;
+    }
+    
+    protected void setPolicyInternal(Policy newPolicy) {
+        this.policy = newPolicy;
+        if (algorithmSuite == null) {
+            Iterator i = policy.getAlternatives();
+            while (i.hasNext() && algorithmSuite == null) {
+                List<PolicyComponent> p = CastUtils.cast((List)i.next());
+                for (PolicyComponent p2 : p) {
+                    if (p2 instanceof Binding) {
+                        algorithmSuite = ((Binding)p2).getAlgorithmSuite();
+                    }
+                }
+            }
+        }
+    }
+    
+    protected void setPolicyInternal(Element newPolicy) {
+        this.setPolicyInternal(bus.getExtension(PolicyBuilder.class).getPolicy(newPolicy));
+    }
+
 
     private void createClient() throws BusException, EndpointException {
         if (client != null) {
@@ -322,7 +404,7 @@ public class STSClient implements Configurable, InterceptorProvider {
                 Conduit conduit = client.getConduit();
                 EffectivePolicy effectivePolicy = pe.getEffectiveClientRequestPolicy(client.getEndpoint()
                     .getEndpointInfo(), boi, conduit);
-                setPolicy(effectivePolicy.getPolicy());
+                setPolicyInternal(effectivePolicy.getPolicy());
                 return boi;
             }
         }
@@ -368,26 +450,44 @@ public class STSClient implements Configurable, InterceptorProvider {
         writer.writeStartElement("wst", "RequestSecurityToken", namespace);
         writer.writeNamespace("wst", namespace);
         boolean wroteKeySize = false;
-        String keyType = null;
+
+        String keyTypeTemplate = null;
+        String sptt = null;
+        
         if (template != null) {
+            if (this.useSecondaryParameters()) {
+                writer.writeStartElement("wst", "SecondaryParameters", namespace);
+            }
+            
             Element tl = DOMUtils.getFirstElement(template);
             while (tl != null) {
                 StaxUtils.copy(tl, writer);
                 if ("KeyType".equals(tl.getLocalName())) {
-                    keyType = DOMUtils.getContent(tl);
+                    keyTypeTemplate = DOMUtils.getContent(tl);
                 } else if ("KeySize".equals(tl.getLocalName())) {
                     wroteKeySize = true;
                     keySize = Integer.parseInt(DOMUtils.getContent(tl));
+                } else if ("TokenType".equals(tl.getLocalName())) {
+                    sptt = DOMUtils.getContent(tl);
                 }
                 tl = DOMUtils.getNextElement(tl);
             }
+            
+            if (this.useSecondaryParameters()) {
+                writer.writeEndElement();
+            }
         }
 
-        writer.writeStartElement("wst", "RequestType", namespace);
-        writer.writeCharacters(namespace + requestType);
-        writer.writeEndElement();
+        addRequestType(requestType, writer);
         addAppliesTo(writer, appliesTo);
-        keyType = writeKeyType(writer, keyType);
+        addOnBehalfOf(writer);
+        if (sptt == null) {
+            addTokenType(writer);
+        }
+        if (keyTypeTemplate == null) {
+            keyTypeTemplate = keyType;
+        }
+        keyTypeTemplate = writeKeyType(writer, keyTypeTemplate);
 
         byte[] requestorEntropy = null;
         X509Certificate cert = null;
@@ -396,52 +496,14 @@ public class STSClient implements Configurable, InterceptorProvider {
         if (keySize <= 0) {
             keySize = 256;
         }
-        if (keyType.endsWith("SymmetricKey")) {
-            if (!wroteKeySize && (!isSecureConv || keySize != 256)) {
-                writer.writeStartElement("wst", "KeySize", namespace);
-                writer.writeCharacters(Integer.toString(keySize));
-                writer.writeEndElement();
-            }
-
-            if (requiresEntropy) {
-                writer.writeStartElement("wst", "Entropy", namespace);
-                writer.writeStartElement("wst", "BinarySecret", namespace);
-                writer.writeAttribute("Type", namespace + "/Nonce");
-                if (algorithmSuite == null) {
-                    requestorEntropy = WSSecurityUtil.generateNonce(keySize / 8);
-                } else {
-                    requestorEntropy = WSSecurityUtil
-                        .generateNonce(algorithmSuite.getMaximumSymmetricKeyLength() / 8);
-                } 
-                writer.writeCharacters(Base64.encode(requestorEntropy));
-
-                writer.writeEndElement();
-                writer.writeEndElement();
-                writer.writeStartElement("wst", "ComputedKeyAlgorithm", namespace);
-                writer.writeCharacters(namespace + "/CK/PSHA1");
-                writer.writeEndElement();
-            }
-        } else if (keyType.endsWith("PublicKey")) {
-            writer.writeStartElement("wst", "UseKey", namespace);
-            writer.writeStartElement("dsig", "KeyInfo", "http://www.w3.org/2000/09/xmldsig#");
-            writer.writeNamespace("dsig", "http://www.w3.org/2000/09/xmldsig#");
-            writer.writeStartElement("dsig", "KeyValue", "http://www.w3.org/2000/09/xmldsig#");
+        if (keyTypeTemplate != null && keyTypeTemplate.endsWith("SymmetricKey")) {
+            requestorEntropy = writeElementsForRSTSymmetricKey(writer, wroteKeySize);
+        } else if (keyTypeTemplate != null && keyTypeTemplate.endsWith("PublicKey")) {
             crypto = createCrypto(false);
             cert = getCert(crypto);
-            PublicKey key = cert.getPublicKey();
-            String pubKeyAlgo = key.getAlgorithm();
-            if ("DSA".equalsIgnoreCase(pubKeyAlgo)) {
-                DSAKeyValue dsaKeyValue = new DSAKeyValue(writer.getDocument(), key);
-                writer.getCurrentNode().appendChild(dsaKeyValue.getElement());
-            } else if ("RSA".equalsIgnoreCase(pubKeyAlgo)) {
-                RSAKeyValue rsaKeyValue = new RSAKeyValue(writer.getDocument(), key);
-                writer.getCurrentNode().appendChild(rsaKeyValue.getElement());
-            }
-
-            writer.writeEndElement();
-            writer.writeEndElement();
-            writer.writeEndElement();
+            writeElementsForRSTPublicKey(writer, cert);
         }
+        
         if (target != null) {
             writer.writeStartElement("wst", "RenewTarget", namespace);
             Element el = target.getUnattachedReference();
@@ -451,6 +513,9 @@ public class STSClient implements Configurable, InterceptorProvider {
             StaxUtils.copy(el, writer);
             writer.writeEndElement();
         }
+        
+        addActAs(writer);
+        
         writer.writeEndElement();
 
         Object obj[] = client.invoke(boi, new DOMSource(writer.getDocument().getDocumentElement()));
@@ -459,9 +524,95 @@ public class STSClient implements Configurable, InterceptorProvider {
         if (cert != null) {
             token.setX509Certificate(cert, crypto);
         }
+        if (token.getTokenType() == null) {
+            if (sptt != null) {
+                token.setTokenType(sptt);
+            } else if (tokenType != null) {
+                token.setTokenType(tokenType);
+            }
+        }
         return token;
     }
     
+    private byte[] writeElementsForRSTSymmetricKey(W3CDOMStreamWriter writer,
+            boolean wroteKeySize) throws Exception {
+        byte[] requestorEntropy = null;
+
+        if (!wroteKeySize && (!isSecureConv || keySize != 256)) {
+            writer.writeStartElement("wst", "KeySize", namespace);
+            writer.writeCharacters(Integer.toString(keySize));
+            writer.writeEndElement();
+        }
+
+        if (requiresEntropy) {
+            writer.writeStartElement("wst", "Entropy", namespace);
+            writer.writeStartElement("wst", "BinarySecret", namespace);
+            writer.writeAttribute("Type", namespace + "/Nonce");
+            if (algorithmSuite == null) {
+                requestorEntropy = WSSecurityUtil.generateNonce(keySize / 8);
+            } else {
+                requestorEntropy = WSSecurityUtil
+                    .generateNonce(algorithmSuite.getMaximumSymmetricKeyLength() / 8);
+            }
+            writer.writeCharacters(Base64.encode(requestorEntropy));
+
+            writer.writeEndElement();
+            writer.writeEndElement();
+            writer.writeStartElement("wst", "ComputedKeyAlgorithm", namespace);
+            writer.writeCharacters(namespace + "/CK/PSHA1");
+            writer.writeEndElement();
+        }
+        return requestorEntropy;
+    }
+
+
+    private void writeElementsForRSTPublicKey(W3CDOMStreamWriter writer,
+            X509Certificate cert) throws Exception {
+        writer.writeStartElement("wst", "UseKey", namespace);
+        writer.writeStartElement("dsig", "KeyInfo", "http://www.w3.org/2000/09/xmldsig#");
+        writer.writeNamespace("dsig", "http://www.w3.org/2000/09/xmldsig#");
+
+        boolean useCert = useCertificateForConfirmationKeyInfo;
+        String useCertStr = (String)getProperty(SecurityConstants.STS_TOKEN_USE_CERT_FOR_KEYINFO);
+        if (useCertStr != null) {
+            useCert = Boolean.parseBoolean(useCertStr);
+        }
+        if (useCert) {
+            X509Data certElem = new X509Data(writer.getDocument());
+            certElem.addCertificate(cert);
+            writer.getCurrentNode().appendChild(certElem.getElement());
+        } else {
+            writer.writeStartElement("dsig", "KeyValue", "http://www.w3.org/2000/09/xmldsig#");
+            PublicKey key = cert.getPublicKey();
+            String pubKeyAlgo = key.getAlgorithm();
+            if ("DSA".equalsIgnoreCase(pubKeyAlgo)) {
+                DSAKeyValue dsaKeyValue = new DSAKeyValue(writer.getDocument(), key);
+                writer.getCurrentNode().appendChild(dsaKeyValue.getElement());
+            } else if ("RSA".equalsIgnoreCase(pubKeyAlgo)) {
+                RSAKeyValue rsaKeyValue = new RSAKeyValue(writer.getDocument(), key);
+                writer.getCurrentNode().appendChild(rsaKeyValue.getElement());
+            }
+            writer.writeEndElement();
+        }
+
+        writer.writeEndElement();
+        writer.writeEndElement();
+    }
+
+    private void addRequestType(String requestType, W3CDOMStreamWriter writer) throws XMLStreamException {
+        writer.writeStartElement("wst", "RequestType", namespace);
+        writer.writeCharacters(namespace + requestType);
+        writer.writeEndElement();
+    }
+    
+    private void addOnBehalfOf(W3CDOMStreamWriter writer) throws XMLStreamException  {
+        if (onBehalfOfElement != null) {
+            writer.writeStartElement("wst", "OnBehalfOf", namespace);
+            StaxUtils.copy(onBehalfOfElement, writer);
+            writer.writeEndElement();
+        }
+    }
+    
     private Element getDocumentElement(DOMSource ds) {
         Node nd = ds.getNode();
         if (nd instanceof Document) {
@@ -498,11 +649,12 @@ public class STSClient implements Configurable, InterceptorProvider {
         ExactlyOne one = new ExactlyOne();
         validatePolicy.addPolicyComponent(one);
         All all = new All();
-        SymmetricBinding binding = new SymmetricBinding();
+        PolicyBuilder pbuilder = bus.getExtension(PolicyBuilder.class);
+        SymmetricBinding binding = new SymmetricBinding(pbuilder);
         all.addAssertion(binding);
         one.addPolicyComponent(all);
         all.addAssertion(getAddressingAssertion());
-        ProtectionToken ptoken = new ProtectionToken();
+        ProtectionToken ptoken = new ProtectionToken(pbuilder);
         binding.setProtectionToken(ptoken);
         binding.setIncludeTimestamp(true);
         binding.setEntireHeadersAndBodySignatures(true);
@@ -565,11 +717,12 @@ public class STSClient implements Configurable, InterceptorProvider {
         ExactlyOne one = new ExactlyOne();
         cancelPolicy.addPolicyComponent(one);
         All all = new All();
-        one.addPolicyComponent(all);
-        SymmetricBinding binding = new SymmetricBinding();
+        one.addPolicyComponent(all); 
+        PolicyBuilder pbuilder = bus.getExtension(PolicyBuilder.class);
+        SymmetricBinding binding = new SymmetricBinding(pbuilder);
         all.addAssertion(binding);
         all.addAssertion(getAddressingAssertion());
-        ProtectionToken ptoken = new ProtectionToken();
+        ProtectionToken ptoken = new ProtectionToken(pbuilder);
         binding.setProtectionToken(ptoken);
         binding.setIncludeTimestamp(true);
         binding.setEntireHeadersAndBodySignatures(true);
@@ -631,23 +784,32 @@ public class STSClient implements Configurable, InterceptorProvider {
             LOG.log(Level.WARNING, "Problem cancelling token", ex);
         }
     }
+    
+    private boolean useSecondaryParameters() {
+        return !STSUtils.WST_NS_05_02.equals(namespace);
+    }
 
-    private String writeKeyType(W3CDOMStreamWriter writer, String keyType) throws XMLStreamException {
+    private String writeKeyType(W3CDOMStreamWriter writer, String keyTypeToWrite) 
+        throws XMLStreamException {
         if (isSecureConv) {
             addLifetime(writer);
-            if (keyType == null) {
+            if (keyTypeToWrite == null) {
                 writer.writeStartElement("wst", "TokenType", namespace);
                 writer.writeCharacters(STSUtils.getTokenTypeSCT(namespace));
                 writer.writeEndElement();
-                keyType = namespace + "/SymmetricKey";
+                keyTypeToWrite = namespace + "/SymmetricKey";
             }
-        } else if (keyType == null) {
+        } else if (keyTypeToWrite == null && sendKeyType) {
             writer.writeStartElement("wst", "KeyType", namespace);
             writer.writeCharacters(namespace + "/SymmetricKey");
             writer.writeEndElement();
-            keyType = namespace + "/SymmetricKey";
+            keyTypeToWrite = namespace + "/SymmetricKey";
+        } else if (keyTypeToWrite != null) {
+            writer.writeStartElement("wst", "KeyType", namespace);
+            writer.writeCharacters(keyTypeToWrite);
+            writer.writeEndElement();
         }
-        return keyType;
+        return keyTypeToWrite;
     }
 
     private X509Certificate getCert(Crypto crypto) throws Exception {
@@ -664,7 +826,12 @@ public class STSClient implements Configurable, InterceptorProvider {
                 throw new Fault("No alias specified for retrieving PublicKey", LOG);
             }
         }
-        return crypto.getCertificates(alias)[0];
+
+        X509Certificate certs[] = crypto.getCertificates(alias);
+        if (certs == null || certs.length == 0) {
+            throw new Fault("Could not get X509Certificate for alias " + alias, LOG);
+        }
+        return certs[0];
     }
 
     private void addLifetime(XMLStreamWriter writer) throws XMLStreamException {
@@ -699,6 +866,43 @@ public class STSClient implements Configurable, InterceptorProvider {
         }
     }
 
+    private void addTokenType(XMLStreamWriter writer) throws XMLStreamException {
+        if (tokenType != null) {
+            writer.writeStartElement("wst", "TokenType", namespace);
+            writer.writeCharacters(tokenType);
+            writer.writeEndElement();
+        }
+    }
+
+    private void addActAs(W3CDOMStreamWriter writer) throws Exception {
+        if (this.actAs != null) {
+            final boolean isString = this.actAs instanceof String;
+            final boolean isElement = this.actAs instanceof Element; 
+            if (isString || isElement) {
+                final Element actAsEl;
+                
+                if (isString) {
+                    final Document acAsDoc =
+                        DOMUtils.readXml(new StringReader((String) this.actAs));
+                    actAsEl = acAsDoc.getDocumentElement();
+                } else {
+                    actAsEl = (Element) this.actAs;
+                }
+                
+                writer.writeStartElement(STSUtils.WST_NS_08_02, "ActAs");
+                
+                // Unlikely to ever be otherwise, but still prudent to check.
+                if (actAsEl.getOwnerDocument() != writer.getDocument()) {
+                    writer.getDocument().adoptNode(actAsEl);
+                }
+                
+                writer.getCurrentNode().appendChild(actAsEl);
+                
+                writer.writeEndElement();
+            }
+        }
+    }
+
     private SecurityToken createSecurityToken(Element el, byte[] requestorEntropy)
         throws WSSecurityException {
 
@@ -715,6 +919,7 @@ public class STSClient implements Configurable, InterceptorProvider {
         Element rpt = null;
         Element lte = null;
         Element entropy = null;
+        String tt = null;
 
         while (el != null) {
             String ln = el.getLocalName();
@@ -731,6 +936,8 @@ public class STSClient implements Configurable, InterceptorProvider {
                     rpt = el;
                 } else if ("Entropy".equals(ln)) {
                     entropy = el;
+                } else if ("TokenType".equals(ln)) {
+                    tt = DOMUtils.getContent(el);
                 }
             }
             el = DOMUtils.getNextElement(el);
@@ -738,12 +945,13 @@ public class STSClient implements Configurable, InterceptorProvider {
         Element rstDec = rst;
         String id = findID(rar, rur, rstDec);
         if (StringUtils.isEmpty(id)) {
-            throw new TrustException(new Message("NO_ID", LOG));
+            throw new TrustException("NO_ID", LOG);
         }
         SecurityToken token = new SecurityToken(id, rstDec, lte);
         token.setAttachedReference(rar);
         token.setUnattachedReference(rur);
         token.setIssuerAddress(location);
+        token.setTokenType(tt);
 
         byte[] secret = null;
 
@@ -764,7 +972,7 @@ public class STSClient implements Configurable, InterceptorProvider {
 
                     secret = processor.getDecryptedBytes();
                 } catch (IOException e) {
-                    throw new TrustException(new Message("ENCRYPTED_KEY_ERROR", LOG), e);
+                    throw new TrustException("ENCRYPTED_KEY_ERROR", LOG, e);
                 }
             } else if (childQname.equals(new QName(namespace, "ComputedKey"))) {
                 // Handle the computed key
@@ -784,11 +992,11 @@ public class STSClient implements Configurable, InterceptorProvider {
                     try {
                         secret = psha1.createKey(requestorEntropy, serviceEntr, 0, length / 8);
                     } catch (ConversationException e) {
-                        throw new TrustException(new Message("DERIVED_KEY_ERROR", LOG), e);
+                        throw new TrustException("DERIVED_KEY_ERROR", LOG, e);
                     }
                 } else {
                     // Service entropy missing
-                    throw new TrustException(new Message("NO_ENTROPY", LOG));
+                    throw new TrustException("NO_ENTROPY", LOG);
                 }
             }
         } else if (requestorEntropy != null) {
@@ -871,7 +1079,17 @@ public class STSClient implements Configurable, InterceptorProvider {
     private String findID(Element rar, Element rur, Element rst) {
         String id = null;
         if (rst != null) {
-            id = this.getIDFromSTR(rst);
+            QName elName = DOMUtils.getElementQName(rst);
+            if (elName.equals(new QName(WSConstants.SAML_NS, "Assertion"))
+                && rst.hasAttributeNS(null, "AssertionID")) {
+                id = rst.getAttributeNS(null, "AssertionID");
+            } else if (elName.equals(new QName(SAML2_NS, "Assertion"))
+                && rst.hasAttributeNS(null, "ID")) {
+                id = rst.getAttributeNS(null, "ID");
+            }
+            if (id == null) {
+                id = this.getIDFromSTR(rst);
+            }
         }
         if (id == null && rar != null) {
             id = this.getIDFromSTR(rar);
@@ -907,50 +1125,50 @@ public class STSClient implements Configurable, InterceptorProvider {
         template = rstTemplate;
     }
     
-    public List<Interceptor> getOutFaultInterceptors() {
+    public List<Interceptor<? extends Message>> getOutFaultInterceptors() {
         if (client != null) {
             return client.getOutFaultInterceptors();
         }
         return outFault;
     }
 
-    public List<Interceptor> getInFaultInterceptors() {
+    public List<Interceptor<? extends Message>> getInFaultInterceptors() {
         if (client != null) {
             return client.getInFaultInterceptors();
         }
         return inFault;
     }
 
-    public List<Interceptor> getInInterceptors() {
+    public List<Interceptor<? extends Message>> getInInterceptors() {
         if (client != null) {
             return client.getInInterceptors();
         }
         return in;
     }
 
-    public List<Interceptor> getOutInterceptors() {
+    public List<Interceptor<? extends Message>> getOutInterceptors() {
         if (client != null) {
             return client.getOutInterceptors();
         }
         return out;
     }
 
-    public void setInInterceptors(List<Interceptor> interceptors) {
+    public void setInInterceptors(List<Interceptor<? extends Message>> interceptors) {
         getInInterceptors().addAll(interceptors);
     }
 
-    public void setInFaultInterceptors(List<Interceptor> interceptors) {
+    public void setInFaultInterceptors(List<Interceptor<? extends Message>> interceptors) {
         getInFaultInterceptors().addAll(interceptors);
     }
 
-    public void setOutInterceptors(List<Interceptor> interceptors) {
+    public void setOutInterceptors(List<Interceptor<? extends Message>> interceptors) {
         getOutInterceptors().addAll(interceptors);
     }
 
-    public void setOutFaultInterceptors(List<Interceptor> interceptors) {
+    public void setOutFaultInterceptors(List<Interceptor<? extends Message>> interceptors) {
         getOutFaultInterceptors().addAll(interceptors);
     }
-    
+        
     public void setFeatures(List<AbstractFeature> f) {
         features = f;
     }
