@@ -123,11 +123,17 @@ public class JAXRSMultipartTest extends AbstractBusClientServerTestBase {
     }
     
     @Test
-    public void testBookJSONFormFiles() throws Exception {
+    public void testBookJSONFormTwoFiles() throws Exception {
         String address = "http://localhost:" + PORT + "/bookstore/books/filesform";
         doAddFormBook(address, "attachmentFormJsonFiles", 200);               
     }
     
+    @Test
+    public void testBookJSONFormOneFile() throws Exception {
+        String address = "http://localhost:" + PORT + "/bookstore/books/fileform";
+        doAddFormBook(address, "attachmentFormJsonFile", 200);               
+    }
+    
     @Test
     public void testBookAsMessageContextAttachment() throws Exception {
         String address = "http://localhost:" + PORT + "/bookstore/books/attachment";
@@ -352,6 +358,34 @@ public class JAXRSMultipartTest extends AbstractBusClientServerTestBase {
         assertTrue(Arrays.equals(image1, image2));
     }
     
+    @Test
+    public void testGetBookJaxbJsonProxy() throws Exception {
+        String address = "http://localhost:" + PORT;
+        MultipartStore client = JAXRSClientFactory.create(address, MultipartStore.class);
+        
+        
+        Map<String, Book> map = client.getBookJaxbJson();
+        List<Book> result = new ArrayList<Book>(map.values());
+        Book jaxb = result.get(0);
+        assertEquals("jaxb", jaxb.getName());
+        assertEquals(1L, jaxb.getId());
+        Book json = result.get(1);
+        assertEquals("json", json.getName());
+        assertEquals(2L, json.getId());
+    }
+    
+    @Test
+    public void testGetBookJaxbJsonProxy2() throws Exception {
+        String address = "http://localhost:" + PORT;
+        MultipartStore client = JAXRSClientFactory.create(address, MultipartStore.class);
+        
+        Map<String, Object> map = client.getBookJaxbJsonObject();
+        List<Object> result = new ArrayList<Object>(map.values());
+        assertEquals(2, result.size());
+        assertTrue(((Attachment)result.get(0)).getContentType().toString().contains("application/xml"));
+        assertTrue(((Attachment)result.get(1)).getContentType().toString().contains("application/json"));
+    }
+    
     @Test
     public void testAddBookJaxbJsonImageAttachments() throws Exception {
         String address = "http://localhost:" + PORT + "/bookstore/books/jaxbimagejson";
@@ -456,7 +490,8 @@ public class JAXRSMultipartTest extends AbstractBusClientServerTestBase {
     @Test
     public void testUploadImageFromForm2() throws Exception {
         File file = 
-            new File(getClass().getResource("/org/apache/cxf/systest/jaxrs/resources/java.jpg").getFile());
+            new File(getClass().getResource("/org/apache/cxf/systest/jaxrs/resources/java.jpg")
+                               .toURI().getPath());
         String address = "http://localhost:" + PORT + "/bookstore/books/formimage2";
         WebClient client = WebClient.create(address);
         HTTPConduit conduit = WebClient.getConfig(client).getHttpConduit();
@@ -471,7 +506,7 @@ public class JAXRSMultipartTest extends AbstractBusClientServerTestBase {
         byte[] image2 = IOUtils.readBytesFromStream(is2);
         assertTrue(Arrays.equals(image1, image2));
         ContentDisposition cd2 = body2.getRootAttachment().getContentDisposition();
-        assertEquals("attachment;filename=java.jpg", cd2.toString());
+        assertEquals("form-data;name=file;filename=java.jpg", cd2.toString());
         assertEquals("java.jpg", cd2.getParameter("filename"));
     }
     
